<?php
/**
 * @copyright  Copyright (C) 2012 Daniel Calviño Sánchez
 * @copyright  Copyright (C) 2016 Daniel Calviño Sánchez
 * @license    GNU Affero General Public License version 3 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * SimpleCustomRouter component helper.
 */
abstract class SimpleCustomRouterHelper {

    /**
     * Returns a list of the actions that can be performed.
     *
     * @return JObject An object with each action as a property, with boolean
     *         values telling whether the current user is authorised or not for
     *         that action.
     */
    public static function getActions() {
        $user = JFactory::getUser();
        $result = new JObject;

        $actions = array(
            'core.admin', 'core.manage', 'core.create', 'core.edit', 'core.delete',
            'simplecustomrouter.test'
        );

        foreach ($actions as $action) {
            $result->set($action, $user->authorise($action, 'com_simplecustomrouter'));
        }

        return $result;
    }

    /**
     * Filter method for query strings.
     *
     * It removes the leading "index.php?", if any.
     *
     * This method will be called by JForm when filtering the form data.
     *
     * @param string $value The query string to filter.
     * @return string The filtered query string.
     */
    public static function filterQuery($value) {
        // If the query starts with "index.php?" remove the "index.php?".
        if (substr($value, 0, strlen("index.php?")) === "index.php?") {
            $value = substr($value, strlen("index.php?"));
        }

        return $value;
    }
}
