<?php
/**
 * @copyright  Copyright (C) 2012 Daniel Calviño Sánchez
 * @copyright  Copyright (C) 2014 Daniel Calviño Sánchez
 * @copyright  Copyright (C) 2015 Daniel Calviño Sánchez
 * @copyright  Copyright (C) 2016 Daniel Calviño Sánchez
 * @license    GNU Affero General Public License version 3 or later; see LICENSE.txt
 */

require_once 'SeleniumJoomlaTestCase.php';

class SimpleCustomRouterTest extends SeleniumJoomlaTestCase {

    public function setUp() {
        parent::setUp();

        $output = file_get_contents($this->baseUrl().'tests/system/backupRestore.php?action=backup');
        if ($output != 'Backup done') {
            echo 'Configuration or database backup failed!', PHP_EOL;
            die();
        }

        echo 'Configuration and database backup done', PHP_EOL;
    }

    public function tearDown() {
        $output = file_get_contents($this->baseUrl().'tests/system/backupRestore.php?action=restore');
        if ($output != 'Restored') {
            echo 'Restore configuration or database failed!', PHP_EOL;
            die();
        }

        echo 'Configuration and database restored', PHP_EOL;

        parent::tearDown();
    }

    public function testParsingAddingAndRemovingRoute() {
        echo 'Ensure that URL rewriting is enabled', PHP_EOL;
        $this->gotoAdmin();
        $this->doAdminLogin();
        $this->jClick('Global Configuration');
        //As URL rewriting depends also on the .htaccess file it is only
        //checked whether it is enabled or not, failing if it is not enabled
        //instead of just enabling it.
        $this->assertTrue($this->isChecked('//input[@name="jform[sef_rewrite]" and @value="1"]'), 'URL Rewriting enabled');

        echo 'Ensure that cache is disabled', PHP_EOL;
        $this->setCache('off');
        $this->setSimpleCustomRouterCache('off', 'off');

        echo 'Ensure that plugin is enabled', PHP_EOL;
        $this->enableSimpleCustomRouterPlugin();

        echo 'Add dummy article for the menu item', PHP_EOL;
        $this->addArticle('Dummy article for the menu item', 'Dummy article for the menu item text');

        echo 'Add menu item to be assigned in the route', PHP_EOL;
        $this->addMenuItemForArticle('Simple custom router test menu item', 'Dummy article for the menu item');
        $itemId = $this->getMenuItemIdByTitle('Simple custom router test menu item');

        echo 'Ensure that there is no page with the path that will be added', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTrue(strpos($this->getTitle(), 'Error: 404') === 0, 'Error 404 before adding route');

        echo 'Add article for the route', PHP_EOL;
        $this->gotoAdmin();
        $this->addArticle('Simple custom router test', 'Simple custom router test text');
        $id = $this->getArticleIdByTitle('Simple custom router test');

        echo 'Add route', PHP_EOL;
        $this->goToSimpleCustomRouterComponent();
        $this->clickToolbar('New');
        $this->setRouterData('simpleCustomRouterTest', 'option=com_content&view=article&id='.$id);
        $this->clickToolbar('Save & Close');

        echo 'Check path again after adding route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTextPresent('Simple custom router test text', 'Article text after adding route');
        //As no item id was assigned, the title will be the site title
        $this->assertTitle($this->cfg->site_name, 'Page title after adding route');

        echo 'Edit route to set the Itemid', PHP_EOL;
        $this->gotoAdmin();
        $this->goToSimpleCustomRouterComponent();
        $this->selectRouteByPath('simpleCustomRouterTest');
        $this->clickToolbar('Edit');
        $this->setRouterData('simpleCustomRouterTest', 'option=com_content&view=article&id='.$id, $itemId);
        $this->clickToolbar('Save & Close');

        echo 'Check path again after editing route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTextPresent('Simple custom router test text', 'Article text after editing route');
        //As the item id assigned belongs to a menu item article, the page title
        //will be set to the article being shown (not the one assigned to the
        //menu item).
        $this->assertTitle('Simple custom router test', 'Page title after editing route');

        echo 'Remove route', PHP_EOL;
        $this->gotoAdmin();
        $this->goToSimpleCustomRouterComponent();
        $this->selectRouteByPath('simpleCustomRouterTest');
        $this->clickToolbar('Delete');

        echo 'Check path again after removing route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTrue(strpos($this->getTitle(), 'Error: 404') === 0, 'Error 404 after removing route');

        echo 'Ensure article is available', PHP_EOL;
        $this->open($this->baseUrl().'index.php?option=com_content&view=article&id='.$id);
        $this->assertTextPresent('Simple custom router test text', 'Article text after adding route');
    }

    public function testParsingAddingAndRemovingRouteWithCache() {
        echo 'Ensure that URL rewriting is enabled', PHP_EOL;
        $this->gotoAdmin();
        $this->doAdminLogin();
        $this->jClick('Global Configuration');
        //As URL rewriting depends also on the .htaccess file it is only
        //checked whether it is enabled or not, failing if it is not enabled
        //instead of just enabling it.
        $this->assertTrue($this->isChecked('//input[@name="jform[sef_rewrite]" and @value="1"]'), 'URL Rewriting enabled');

        //Use plugin cache instead of global cache to ensure that there are no
        //problems due to other components cache (for example, the title in
        //article pages is cached when the page is first shown, and it won't
        //change even if the page is loaded with another Itemid).
        echo 'Ensure that cache is enabled', PHP_EOL;
        $this->setCache('off');
        $this->setSimpleCustomRouterCache('on', 'on');

        echo 'Ensure that plugin is enabled', PHP_EOL;
        $this->enableSimpleCustomRouterPlugin();

        echo 'Add dummy article for the menu item', PHP_EOL;
        $this->addArticle('Dummy article for the menu item', 'Dummy article for the menu item text');

        echo 'Add menu item to be assigned in the route', PHP_EOL;
        $this->addMenuItemForArticle('Simple custom router test menu item', 'Dummy article for the menu item');
        $itemId = $this->getMenuItemIdByTitle('Simple custom router test menu item');

        echo 'Ensure that there is no page with the path that will be added', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTrue(strpos($this->getTitle(), 'Error: 404') === 0, 'Error 404 before adding route');

        echo 'Add article for the route', PHP_EOL;
        $this->gotoAdmin();
        $this->addArticle('Simple custom router test', 'Simple custom router test text');
        $id = $this->getArticleIdByTitle('Simple custom router test');

        echo 'Add route', PHP_EOL;
        $this->goToSimpleCustomRouterComponent();
        $this->clickToolbar('New');
        $this->setRouterData('simpleCustomRouterTest', 'option=com_content&view=article&id='.$id);
        $this->clickToolbar('Save & Close');

        //This will add the path to the cache
        echo 'Check path again after adding route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTextPresent('Simple custom router test text', 'Article text after adding route');
        //As no item id was assigned, the title will be the site title
        $this->assertTitle($this->cfg->site_name, 'Page title after adding route');

        //This should use the cache
        echo 'Check path again after adding route (cache)', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTextPresent('Simple custom router test text', 'Article text after adding route (cache)');
        $this->assertTitle($this->cfg->site_name, 'Page title after adding route (cache)');

        //This should clean the cache
        echo 'Edit route to set the Itemid', PHP_EOL;
        $this->gotoAdmin();
        $this->goToSimpleCustomRouterComponent();
        $this->selectRouteByPath('simpleCustomRouterTest');
        $this->clickToolbar('Edit');
        $this->setRouterData('simpleCustomRouterTest', 'option=com_content&view=article&id='.$id, $itemId);
        $this->clickToolbar('Save & Close');

        //This will add the path to the cache again
        echo 'Check path again after editing route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTextPresent('Simple custom router test text', 'Article text after editing route');
        //As the item id assigned belongs to a menu item article, the page title
        //will be set to the article being shown (not the one assigned to the
        //menu item).
        $this->assertTitle('Simple custom router test', 'Page title after editing route');

        //This should use the cache
        echo 'Check path again after editing route (cache)', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTextPresent('Simple custom router test text', 'Article text after editing route (cache)');
        $this->assertTitle('Simple custom router test', 'Page title after editing route (cache)');

        //This should clean the cache
        echo 'Edit route path', PHP_EOL;
        $this->gotoAdmin();
        $this->goToSimpleCustomRouterComponent();
        $this->selectRouteByPath('simpleCustomRouterTest');
        $this->clickToolbar('Edit');
        $this->setRouterData('simpleCustomRouterTestNew', 'option=com_content&view=article&id='.$id, $itemId);
        $this->clickToolbar('Save & Close');

        //The old path should have been removed from cache
        echo 'Check old path again after editing route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest');
        $this->assertTrue(strpos($this->getTitle(), 'Error: 404') === 0, 'Error 404 for old path after editing route');

        //This will add the new path to the cache
        echo 'Check new path after editing route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTestNew');
        $this->assertTextPresent('Simple custom router test text', 'Article text for new path after editing route');
        //As the item id assigned belongs to a menu item article, the page title
        //will be set to the article being shown (not the one assigned to the
        //menu item).
        $this->assertTitle('Simple custom router test', 'Page title for new path after editing route');

        //This should use the cache
        echo 'Check new path after editing route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTestNew');
        $this->assertTextPresent('Simple custom router test text', 'Article text for new path after editing route (cache)');
        $this->assertTitle('Simple custom router test', 'Page title for new path after editing route (cache)');

        //This should clean the cache
        echo 'Remove route', PHP_EOL;
        $this->gotoAdmin();
        $this->goToSimpleCustomRouterComponent();
        $this->selectRouteByPath('simpleCustomRouterTestNew');
        $this->clickToolbar('Delete');

        //The new path should have been removed from cache
        echo 'Check new path again after removing route', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTestNew');
        $this->assertTrue(strpos($this->getTitle(), 'Error: 404') === 0, 'Error 404 for new path after removing route');

        echo 'Ensure article is available', PHP_EOL;
        $this->open($this->baseUrl().'index.php?option=com_content&view=article&id='.$id);
        $this->assertTextPresent('Simple custom router test text', 'Article text after removing route');
    }

    public function testTestRoute() {
        echo 'Ensure that URL rewriting is enabled', PHP_EOL;
        $this->gotoAdmin();
        $this->doAdminLogin();
        $this->jClick('Global Configuration');
        //As URL rewriting depends also on the .htaccess file it is only
        //checked whether it is enabled or not, failing if it is not enabled
        //instead of just enabling it.
        $this->assertTrue($this->isChecked('//input[@name="jform[sef_rewrite]" and @value="1"]'), 'URL Rewriting enabled');

        echo 'Ensure that cache is disabled', PHP_EOL;
        $this->setCache('off');
        $this->setSimpleCustomRouterCache('off', 'off');

        echo 'Ensure that plugin is enabled', PHP_EOL;
        $this->enableSimpleCustomRouterPlugin();

        echo 'Ensure that there is no page with the paths that will be added', PHP_EOL;
        $this->open($this->baseUrl().'simpleCustomRouterTest-42');
        $this->assertTrue(strpos($this->getTitle(), 'Error: 404') === 0, 'Error 404 before adding route');
        $this->open($this->baseUrl().'simpleCustomRouterTestNew');
        $this->assertTrue(strpos($this->getTitle(), 'Error: 404') === 0, 'Error 404 before adding route');

        echo 'Check initial state of Test routes form', PHP_EOL;
        $this->gotoAdmin();
        $this->goToSimpleCustomRouterComponent();
        $this->clickToolbar('Test routes');
        $this->assertEquals('', $this->getText('id=path'), 'Path for initial Test routes form');
        $this->assertEquals('', $this->getText('id=generated-query'), 'Generated query for initial Test routes form');
        $this->assertEquals('', $this->getText('id=query'), 'Query for initial Test routes form');
        $this->assertEquals('', $this->getText('id=generated-path'), 'Generated path for initial Test routes form');

        echo 'Test route not added', PHP_EOL;
        $this->testParse('simpleCustomRouterTest-42');
        $this->assertEquals('[No query was generated for the tested path]', $this->getText('id=generated-query'), 'Generated query for route not added');
        $this->testBuild('option=com_content&view=article&id=42');
        $this->assertEquals('[No path was generated for the tested query]', $this->getText('id=generated-path'), 'Generated path for route not added');
        $this->clickToolbar('Close');

        echo 'Add first route', PHP_EOL;
        $this->clickToolbar('New');
        $this->setRouterData('simpleCustomRouterTest-(\d+)', 'option=com_content&view=article&id={1}');
        $this->clickToolbar('Save & Close');

        echo 'Test after adding first route', PHP_EOL;
        $this->clickToolbar('Test routes');
        $this->testParse('simpleCustomRouterTest-42');
        $this->assertEquals('option=com_content&view=article&id=42', $this->getText('id=generated-query'), 'Generated query after adding first route');
        $this->testBuild('option=com_content&view=article&id=42');
        $this->assertEquals('simpleCustomRouterTest-42', $this->getText('id=generated-path'), 'Generated path after adding first route');
        $this->clickToolbar('Close');

        echo 'Test with extra query parameters after adding first route', PHP_EOL;
        $this->clickToolbar('Test routes');
        $this->testParse('simpleCustomRouterTest-42?unknownParameter=108');
        $this->assertEquals('option=com_content&view=article&id=42&unknownParameter=108', $this->getText('id=generated-query'), 'Generated query with extra parameters after adding first route');
        $this->testBuild('option=com_content&unknownParameter=108&view=article&id=42');
        $this->assertEquals('simpleCustomRouterTest-42?unknownParameter=108', $this->getText('id=generated-path'), 'Generated path with extra parameters after adding first route');
        $this->clickToolbar('Close');

        //The second route will override the first when building with id=42
        echo 'Add second route', PHP_EOL;
        $this->clickToolbar('New');
        $this->setRouterData('simpleCustomRouterTestNew', 'option=com_content&view=article&id=42');
        $this->clickToolbar('Save & Close');

        echo 'Test after adding second route', PHP_EOL;
        $this->clickToolbar('Test routes');
        $this->testParse('simpleCustomRouterTestNew');
        $this->assertEquals('option=com_content&view=article&id=42', $this->getText('id=generated-query'), 'Generated query for second route after adding second route');
        $this->testParse('simpleCustomRouterTest-42');
        $this->assertEquals('option=com_content&view=article&id=42', $this->getText('id=generated-query'), 'Generated query for first route after adding second route');
        //Check that the second route overrides the first
        $this->testBuild('option=com_content&view=article&id=42');
        $this->assertEquals('simpleCustomRouterTestNew', $this->getText('id=generated-path'), 'Generated path for second route after adding second route');
        //Check that the first route is still working
        $this->testBuild('option=com_content&view=article&id=108');
        $this->assertEquals('simpleCustomRouterTest-108', $this->getText('id=generated-path'), 'Generated path for first route after adding second route');
        $this->clickToolbar('Close');
    }

    public function testSortRoutes() {
        //To sort the routes in the administrator view there is no need to
        //ensure that URL rewriting is enabled, add articles...

        echo 'Ensure that there are no routes with the paths and queries that will be added', PHP_EOL;
        $this->gotoAdmin();
        $this->doAdminLogin();
        $this->goToSimpleCustomRouterComponent();
        if ($this->getSelectedLabel('limit') != 'All') {
            $this->select('limit', 'All');
            $this->waitForPageToLoad('30000');
        }
        $this->assertTextNotPresent('path2-id1-query3', 'First route path before adding routes to sort');
        $this->assertTextNotPresent('option=com_simplecustomroutertest&id=3', 'First route query before adding routes to sort');
        $this->assertTextNotPresent('path1-id2-query1', 'Second route path before adding routes to sort');
        $this->assertTextNotPresent('option=com_simplecustomroutertest&id=1', 'Second route query before adding routes to sort');
        $this->assertTextNotPresent('path3-id3-query2', 'Third route path before adding routes to sort');
        $this->assertTextNotPresent('option=com_simplecustomroutertest&id=2', 'Third route query before adding routes to sort');

        echo 'Add first route', PHP_EOL;
        $this->clickToolbar('New');
        $this->setRouterData('path2-id1-query3', 'option=com_simplecustomroutertest&id=3');
        $this->clickToolbar('Save & New');

        echo 'Add second route', PHP_EOL;
        $this->setRouterData('path1-id2-query1', 'option=com_simplecustomroutertest&id=1');
        $this->clickToolbar('Save & New');

        echo 'Add third route', PHP_EOL;
        $this->setRouterData('path3-id3-query2', 'option=com_simplecustomroutertest&id=2');
        $this->clickToolbar('Save & Close');

        //By default, the routes are sorted by their ID in ascending order, so
        //sorting by the ID column will sort in descending order.
        echo 'Sort by ID column in descending order using table header', PHP_EOL;
        $this->sortByColumnHeader('ID');
        $this->assertEquals('ID', $this->getSelectedLabel('sortTable'), 'Sort order sorting by descending ID using table header');
        $this->assertEquals('Descending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by descending ID using table header');
        $this->assertPathBeforePath('path3-id3-query2', 'path1-id2-query1', 'Third route appears before second route sorting by descending ID using table header');
        $this->assertPathBeforePath('path1-id2-query1', 'path2-id1-query3', 'Second route appears before first route sorting by descending ID using table header');

        echo 'Sort by ID column in ascending order using table header', PHP_EOL;
        $this->sortByColumnHeader('ID');
        $this->assertEquals('ID', $this->getSelectedLabel('sortTable'), 'Sort order sorting by ascending ID using table header');
        $this->assertEquals('Ascending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by ascending ID using table header');
        $this->assertPathBeforePath('path2-id1-query3', 'path1-id2-query1', 'First route appears before second route sorting by ascending ID using table header');
        $this->assertPathBeforePath('path1-id2-query1', 'path3-id3-query2', 'Second route appears before third route sorting by ascending ID using table header');

        echo 'Sort by path column in ascending order using table header', PHP_EOL;
        $this->sortByColumnHeader('Path');
        $this->assertEquals('Path', $this->getSelectedLabel('sortTable'), 'Sort order sorting by ascending path using table header');
        $this->assertEquals('Ascending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by ascending path using table header');
        $this->assertPathBeforePath('path1-id2-query1', 'path2-id1-query3', 'Second route appears before first route sorting by ascending path using table header');
        $this->assertPathBeforePath('path2-id1-query3', 'path3-id3-query2', 'First route appears before third route sorting by ascending path using table header');

        echo 'Sort by path column in descending order using table header', PHP_EOL;
        $this->sortByColumnHeader('Path');
        $this->assertEquals('Path', $this->getSelectedLabel('sortTable'), 'Sort order sorting by descending path using table header');
        $this->assertEquals('Descending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by descending path using table header');
        $this->assertPathBeforePath('path3-id3-query2', 'path2-id1-query3', 'Third route appears before first route sorting by descending path using table header');
        $this->assertPathBeforePath('path2-id1-query3', 'path1-id2-query1', 'First route appears before second route sorting by descending path using table header');

        echo 'Sort by query column in ascending order using table header', PHP_EOL;
        $this->sortByColumnHeader('Query');
        $this->assertEquals('Query', $this->getSelectedLabel('sortTable'), 'Sort order sorting by ascending query using table header');
        $this->assertEquals('Ascending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by ascending query using table header');
        $this->assertPathBeforePath('path1-id2-query1', 'path3-id3-query2', 'Second route appears before third route sorting by ascending query using table header');
        $this->assertPathBeforePath('path3-id3-query2', 'path2-id1-query3', 'Third route appears before first route sorting by ascending query using table header');

        echo 'Sort by query column in descending order using table header', PHP_EOL;
        $this->sortByColumnHeader('Query');
        $this->assertEquals('Query', $this->getSelectedLabel('sortTable'), 'Sort order sorting by descending query using table header');
        $this->assertEquals('Descending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by descending query using table header');
        $this->assertPathBeforePath('path2-id1-query3', 'path3-id3-query2', 'First route appears before third route sorting by descending query using table header');
        $this->assertPathBeforePath('path3-id3-query2', 'path1-id2-query1', 'Third route appears before second route sorting by descending query using table header');

        echo 'Sort by ID column in ascending order using toolbar', PHP_EOL;
        $this->sortByToolbar('ID');
        $this->assertEquals('ID', $this->getSelectedLabel('sortTable'), 'Sort order sorting by ascending ID using toolbar');
        $this->assertEquals('Ascending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by ascending ID using toolbar');
        $this->assertPathBeforePath('path2-id1-query3', 'path1-id2-query1', 'First route appears before second route sorting by ascending ID using toolbar');
        $this->assertPathBeforePath('path1-id2-query1', 'path3-id3-query2', 'Second route appears before third route sorting by ascending ID using toolbar');

        echo 'Sort by ID column in descending order using toolbar', PHP_EOL;
        $this->sortByToolbarDirection('Descending');
        $this->assertEquals('ID', $this->getSelectedLabel('sortTable'), 'Sort order sorting by descending ID using toolbar');
        $this->assertEquals('Descending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by descending ID using toolbar');
        $this->assertPathBeforePath('path3-id3-query2', 'path1-id2-query1', 'Third route appears before second route sorting by descending ID using toolbar');
        $this->assertPathBeforePath('path1-id2-query1', 'path2-id1-query3', 'Second route appears before first route sorting by descending ID using toolbar');

        echo 'Sort by path column in ascending order using toolbar', PHP_EOL;
        $this->sortByToolbar('Path');
        $this->assertEquals('Path', $this->getSelectedLabel('sortTable'), 'Sort order sorting by ascending path using toolbar');
        $this->assertEquals('Ascending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by ascending path using toolbar');
        $this->assertPathBeforePath('path1-id2-query1', 'path2-id1-query3', 'Second route appears before first route sorting by ascending path using toolbar');
        $this->assertPathBeforePath('path2-id1-query3', 'path3-id3-query2', 'First route appears before third route sorting by ascending path using toolbar');

        echo 'Sort by path column in descending order using toolbar', PHP_EOL;
        $this->sortByToolbarDirection('Descending');
        $this->assertEquals('Path', $this->getSelectedLabel('sortTable'), 'Sort order sorting by descending path using toolbar');
        $this->assertEquals('Descending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by descending path using toolbar');
        $this->assertPathBeforePath('path3-id3-query2', 'path2-id1-query3', 'Third route appears before first route sorting by descending path using toolbar');
        $this->assertPathBeforePath('path2-id1-query3', 'path1-id2-query1', 'First route appears before second route sorting by descending path using toolbar');

        echo 'Sort by path column in ascending order again using toolbar', PHP_EOL;
        $this->sortByToolbarDirection('Ascending');
        $this->assertEquals('Path', $this->getSelectedLabel('sortTable'), 'Sort order sorting by ascending path again using toolbar');
        $this->assertEquals('Ascending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by ascending path again using toolbar');
        $this->assertPathBeforePath('path1-id2-query1', 'path2-id1-query3', 'Second route appears before first route sorting by ascending path again using toolbar');
        $this->assertPathBeforePath('path2-id1-query3', 'path3-id3-query2', 'First route appears before third route sorting by ascending path again using toolbar');

        echo 'Sort by query column in ascending order using toolbar', PHP_EOL;
        $this->sortByToolbar('Query');
        $this->assertEquals('Query', $this->getSelectedLabel('sortTable'), 'Sort order sorting by ascending query using toolbar');
        $this->assertEquals('Ascending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by ascending query using toolbar');
        $this->assertPathBeforePath('path1-id2-query1', 'path3-id3-query2', 'Second route appears before third route sorting by ascending query using toolbar');
        $this->assertPathBeforePath('path3-id3-query2', 'path2-id1-query3', 'Third route appears before first route sorting by ascending query using toolbar');

        echo 'Sort by query column in descending order using toolbar', PHP_EOL;
        $this->sortByToolbarDirection('Descending');
        $this->assertEquals('Query', $this->getSelectedLabel('sortTable'), 'Sort order sorting by descending query using toolbar');
        $this->assertEquals('Descending', $this->getSelectedLabel('directionTable'), 'Sort direction sorting by descending query using toolbar');
        $this->assertPathBeforePath('path2-id1-query3', 'path3-id3-query2', 'First route appears before third route sorting by descending query using toolbar');
        $this->assertPathBeforePath('path3-id3-query2', 'path1-id2-query1', 'Third route appears before second route sorting by descending query using toolbar');
    }

    public function testSearchRoutes() {
        //To search the routes in the administrator view there is no need to
        //ensure that URL rewriting is enabled, add articles...

        echo 'Ensure that there are no routes with the paths and queries that will be added', PHP_EOL;
        $this->gotoAdmin();
        $this->doAdminLogin();
        $this->goToSimpleCustomRouterComponent();
        if ($this->getSelectedLabel('limit') != 'All') {
            $this->select('limit', 'All');
            $this->waitForPageToLoad('30000');
        }
        $this->assertTextNotPresent('SimpleCustomRouterTest/aPathYouAreNotLikelyToHave', 'First route path before adding routes to search');
        $this->assertTextNotPresent('option=com_simplecustomroutertest&id=aQueryYouAreNotLikelyToHave', 'First route query before adding routes to search');
        $this->assertTextNotPresent('SimpleCustomRouterTest/aPathYouAreUnlikelyToHave', 'Second route path before adding routes to search');
        $this->assertTextNotPresent('option=com_simplecustomroutertest&id=aQueryYouAreUnlikelyToHave', 'Second route query before adding routes to search');
        $this->assertTextNotPresent('SimpleCustomRouterTest/anotherPathYouAreNotLikelyToHave', 'Third route path before adding routes to search');
        $this->assertTextNotPresent('option=com_simplecustomroutertest&id=anotherQueryYouAreNotLikelyToHave', 'Third route query before adding routes to search');

        echo 'Add first route', PHP_EOL;
        $this->clickToolbar('New');
        $this->setRouterData('SimpleCustomRouterTest/aPathYouAreNotLikelyToHave', 'option=com_simplecustomroutertest&id=aQueryYouAreNotLikelyToHave');
        $this->clickToolbar('Save & New');

        echo 'Add second route', PHP_EOL;
        $this->setRouterData('SimpleCustomRouterTest/aPathYouAreUnlikelyToHave', 'option=com_simplecustomroutertest&id=aQueryYouAreUnlikelyToHave');
        $this->clickToolbar('Save & New');

        echo 'Add third route', PHP_EOL;
        $this->setRouterData('SimpleCustomRouterTest/anotherPathYouAreNotLikelyToHave', 'option=com_simplecustomroutertest&id=anotherQueryYouAreNotLikelyToHave');
        $this->clickToolbar('Save & Close');

        $route2Id = $this->getRouteIdByPath('SimpleCustomRouterTest/aPathYouAreUnlikelyToHave');

        echo 'Search by path or query', PHP_EOL;
        //As the default filter search is being used, the search can be made
        //using the method defined in SeleniumJoomlaTestCase.
        $this->filterView('YouAreNotLikelyToHave');
        $this->assertElementValueEquals('filter_search', 'YouAreNotLikelyToHave', 'Filter searching by path or query');
        $this->assertEquals(2, $this->getNumberOfRoutes(), 'Number of routes searching by path or query');
        $this->assertEquals(0, $this->getRouteIndexByPath('SimpleCustomRouterTest/aPathYouAreNotLikelyToHave'), 'First route index searching by path or query');
        $this->assertEquals(1, $this->getRouteIndexByPath('SimpleCustomRouterTest/anotherPathYouAreNotLikelyToHave'), 'Third route index searching by path or query');

        echo 'Clean the search', PHP_EOL;
        $this->click('//i[@class="icon-remove"]/..');
        $this->waitForPageToLoad(30000);
        $this->assertElementValueEquals('filter_search', '', 'Filter after cleaning the search');
        $this->assertGreaterThanOrEqual(3, $this->getNumberOfRoutes(), 'Number of routes after cleaning the search');

        echo 'Search by path or query (case insensitive)', PHP_EOL;
        $this->filterView('youarenotlikelytohave');
        $this->assertElementValueEquals('filter_search', 'youarenotlikelytohave', 'Filter searching by path or query (case insensitive)');
        $this->assertEquals(2, $this->getNumberOfRoutes(), 'Number of routes searching by path or query (case insensitive)');
        $this->assertEquals(0, $this->getRouteIndexByPath('SimpleCustomRouterTest/aPathYouAreNotLikelyToHave'), 'First route index searching by path or query (case insensitive)');
        $this->assertEquals(1, $this->getRouteIndexByPath('SimpleCustomRouterTest/anotherPathYouAreNotLikelyToHave'), 'Third route index searching by path or query (case insensitive)');

        echo 'Search by path', PHP_EOL;
        $this->filterView('aPathYouAre');
        $this->assertElementValueEquals('filter_search', 'aPathYouAre', 'Filter searching by path');
        $this->assertEquals(2, $this->getNumberOfRoutes(), 'Number of routes searching by path');
        $this->assertEquals(0, $this->getRouteIndexByPath('SimpleCustomRouterTest/aPathYouAreNotLikelyToHave'), 'First route index searching by path');
        $this->assertEquals(1, $this->getRouteIndexByPath('SimpleCustomRouterTest/aPathYouAreUnlikelyToHave'), 'Second route index searching by path');

        echo 'Search by query', PHP_EOL;
        $this->filterView('anotherQueryYouAreNotLikelyToHave');
        $this->assertElementValueEquals('filter_search', 'anotherQueryYouAreNotLikelyToHave', 'Filter searching by query');
        $this->assertEquals(1, $this->getNumberOfRoutes(), 'Number of routes searching by query');
        $this->assertEquals(0, $this->getRouteIndexByPath('SimpleCustomRouterTest/anotherPathYouAreNotLikelyToHave'), 'Third route index searching by query');

        echo 'Search by ID', PHP_EOL;
        $this->filterView('ID:'.$route2Id);
        $this->assertElementValueEquals('filter_search', 'ID:'.$route2Id, 'Filter searching by ID');
        $this->assertEquals(1, $this->getNumberOfRoutes(), 'Number of routes searching by ID');
        $this->assertEquals(0, $this->getRouteIndexByPath('SimpleCustomRouterTest/aPathYouAreUnlikelyToHave'), 'Second route index searching by ID');

        echo 'Search with no results', PHP_EOL;
        $this->filterView('aPathYouAreNotLikelyToHaveThatWasNotAddedInTheTestsEither');
        $this->assertEquals(0, $this->getXpathCount('//table[@id="routeList"]'), 'Route list is not shown when search has no results');
        $this->assertTextPresent('There are no routes matching your search', 'Message is shown when search has no results');
    }

    public function testWarnDisabledPlugin() {
        echo 'Ensure that plugin is enabled', PHP_EOL;
        $this->gotoAdmin();
        $this->doAdminLogin();
        $this->enableSimpleCustomRouterPlugin();

        echo 'Ensure that no warning is shown in the administrator interface', PHP_EOL;
        $this->goToSimpleCustomRouterComponent();
        $this->assertTextNotPresent('Simple Custom Router plugin', '"Plugin not enabled" alert when plugin is enabled');

        echo 'Disable plugin', PHP_EOL;
        $this->disableSimpleCustomRouterPlugin();

        echo 'Ensure that a warning is shown in the administrator interface', PHP_EOL;
        $this->goToSimpleCustomRouterComponent();
        $this->assertTextPresent('Simple Custom Router plugin', '"Plugin not enabled" alert when plugin is not enabled');

        echo 'Enable plugin again through warning link', PHP_EOL;
        $this->click('link=enable this plugin');
        $this->waitForPageToLoad('30000');
        $this->assertEquals(1, $this->getXpathCount('//body[contains(@class, "com_plugins")]'), 'Current page is plugin manager');
        // Before Joomla! 3.4, the id "articleList" was used instead of
        // "pluginList".
        $this->assertEquals('Simple Custom Router', $this->getText('//table[@id="articleList" or @id="pluginList"]/tbody//tr/td[position() = 4]//a'), 'First element in plugin list is Simple Custom Router');
        $this->enableFirstPluginInPluginManagerList();

        echo 'Ensure that no warning is shown in the administrator interface', PHP_EOL;
        $this->goToSimpleCustomRouterComponent();
        $this->assertTextNotPresent('Simple Custom Router plugin', '"Plugin not enabled" alert when plugin is enabled again');
    }

    public function testSaveInvalidRoute() {
        //To save the routes in the administrator view there is no need to
        //ensure that URL rewriting is enabled, add articles...

        echo 'Try to add a route without a path nor a query', PHP_EOL;
        $this->gotoAdmin();
        $this->doAdminLogin();
        $this->goToSimpleCustomRouterComponent();
        $this->clickToolbar('New');
        $this->setRouterData('', '');
        $this->clickToolbar('Save & Close');
        $this->assertTextPresent('Simple Custom Router Manager: New route', 'Current page is the "New route" page');
        $this->assertTextPresent('Field required: Path', 'Warning about path being required is shown');
        $this->assertTextPresent('Field required: Query', 'Warning about query being required is shown');

        echo 'Try to add a route without a path', PHP_EOL;
        $this->setRouterData('', 'option=com_simplecustomroutertest&id=aQueryYouAreNotLikelyToHave');
        $this->clickToolbar('Save & Close');
        $this->assertTextPresent('Simple Custom Router Manager: New route', 'Current page is the "New route" page');
        $this->assertTextPresent('Field required: Path', 'Warning about path being required is shown');
        $this->assertTextNotPresent('Field required: Query', 'Warning about query being required is not shown');
        $this->assertElementValueEquals('jform_query', 'option=com_simplecustomroutertest&id=aQueryYouAreNotLikelyToHave', 'Query field value is the same introduced before trying to save');

        echo 'Try to add a route without a query', PHP_EOL;
        $this->setRouterData('SimpleCustomRouterTest/aPathYouAreNotLikelyToHave', '');
        $this->clickToolbar('Save & Close');
        $this->assertTextPresent('Simple Custom Router Manager: New route', 'Current page is the "New route" page');
        $this->assertTextNotPresent('Field required: Path', 'Warning about path being required is not shown');
        $this->assertTextPresent('Field required: Query', 'Warning about query being required is shown');
        $this->assertElementValueEquals('jform_path', 'SimpleCustomRouterTest/aPathYouAreNotLikelyToHave', 'Query field value is the same introduced before trying to save');

        echo 'Add a route with a query starting by "index.php?"', PHP_EOL;
        $this->setRouterData('SimpleCustomRouterTest/aPathYouAreNotLikelyToHave', 'index.php?option=com_simplecustomroutertest&id=aQueryYouAreNotLikelyToHave');
        $this->clickToolbar('Save & Close');
        $this->assertTextPresent('Simple Custom Router Manager: Routes', 'Current page is the main "Routes" page');
        $savedQuery = $this->getRouteQueryByPath('SimpleCustomRouterTest/aPathYouAreNotLikelyToHave');
        $this->assertEquals('option=com_simplecustomroutertest&id=aQueryYouAreNotLikelyToHave', $savedQuery, 'Saved query value should not include leading "index.php?"');
    }

    //TODO How to test building routes?

    private function baseUrl() {
        $configuration = new SeleniumConfig();
        return $configuration->host . $configuration->path;
    }

    private function goToPluginManagerAndSearchForSimpleCustomRouter() {
        $this->click('link=regexp:(Plug-?in Manager)|(Plugins)');
        $this->waitForPageToLoad('30000');

        $this->type('filter_search', 'Simple Custom Router');
        $this->click('//*[@class="icon-search"]/ancestor::button');
        $this->waitForPageToLoad('30000');
    }

    private function setSimpleCustomRouterCache($overrideGlobalCache, $enableCache) {
        $this->goToPluginManagerAndSearchForSimpleCustomRouter();

        // Before Joomla! 3.4, the id "articleList" was used instead of
        // "pluginList".
        $this->click('//table[@id="articleList" or @id="pluginList"]/tbody//tr/td[position() = 4]//a');
        $this->waitForPageToLoad('30000');

        if ($overrideGlobalCache == 'on') {
            $this->click('jform_params_overrideDefaultConfiguration1');
        } else {
            $this->click('jform_params_overrideDefaultConfiguration0');
        }

        if ($enableCache == 'on') {
            $this->click('jform_params_enableCache1');
        } else {
            $this->click('jform_params_enableCache0');
        }

        $this->clickToolbar('Save & Close');
    }

    private function enableSimpleCustomRouterPlugin() {
        $this->goToPluginManagerAndSearchForSimpleCustomRouter();

        $this->enableFirstPluginInPluginManagerList();
    }

    private function enableFirstPluginInPluginManagerList() {
        $this->click('//input[@id="cb0"]');
        $this->click('//div[@id="toolbar-publish"]/button');
        $this->waitForPageToLoad('30000');
    }

    private function disableSimpleCustomRouterPlugin() {
        $this->goToPluginManagerAndSearchForSimpleCustomRouter();

        $this->click('//input[@id="cb0"]');
        $this->click('//div[@id="toolbar-unpublish"]/button');
        $this->waitForPageToLoad('30000');
    }

    private function addArticle($title, $text) {
        $this->click('link=Add New Article');
        $this->waitForPageToLoad('30000');
        $this->type('jform_title', $title);
        $this->click('link=Toggle editor');
        $this->type('jform_articletext', $text);
        $this->clickToolbar('Save & Close');
    }

    private function clickToolbar($action) {
        $this->click('//div[@id="toolbar"]//button/text()[normalize-space() = "'.$action.'"]');
        $this->waitForPageToLoad('30000');
    }

    private function getArticleIdByTitle($title) {
        $this->type('filter_search', $title);
        $this->click('//*[@class="icon-search"]/ancestor::button');
        $this->waitForPageToLoad('30000');
        return $this->getText('//table[@id="articleList"]/tbody//tr/td[position() = (count(//table[@id="articleList"]/thead//tr/th//text()[normalize-space() = "ID"]/ancestor::th/preceding-sibling::th) + 1)]');
    }

    private function addMenuItemForArticle($title, $articleTitle) {
        $this->click('link=Main Menu');
        $this->waitForPageToLoad('30000');
        $this->clickToolbar('New');

        //Select the menu item type
        $this->click('//label/text()[normalize-space() = "Menu Item Type"]/ancestor::div[normalize-space(@class)="control-group"]//a/text()[normalize-space() = "Select"]');
        $this->waitforElement('//div[contains(@id, "sbox-content") or contains(@id, "menuTypeModal")]');
        $this->click('//a/text()[normalize-space() = "Single Article"]');
        $this->waitForPageToLoad('30000');

        //Set menu item title
        $this->type('jform_title', $title);

        //Set article for the menu item
        $this->click('//label/text()[normalize-space() = "Select Article"]/ancestor::div[normalize-space(@class)="control-group"]//a/text()[normalize-space() = "Select"]');
        $this->waitforElement('//iframe[contains(@src, "jSelectArticle")]');
        $this->type('filter_search', $articleTitle);
        $this->click('//*[@class="icon-search"]/ancestor::button');
        $this->waitForPageToLoad('30000');
        $this->click('link='.$articleTitle);

        $this->clickToolbar('Save & Close');
    }

    private function getMenuItemIdByTitle($title) {
        $this->type('filter_search', $title);
        $this->click('//*[@class="icon-search"]/ancestor::button');
        $this->waitForPageToLoad('30000');
        return $this->getText('//table[@id="itemList"]/tbody//tr/td[position() = (count(//table[@id="itemList"]/thead//tr/th//text()[normalize-space() = "ID"]/ancestor::th/preceding-sibling::th) + 1)]');
    }

    private function goToSimpleCustomRouterComponent() {
        $this->click('link=Simple Custom Router');
        $this->waitForPageToLoad('30000');
    }

    private function setRouterData($path, $query, $itemId = '') {
        $this->type('jform_path', $path);
        $this->type('jform_query', $query);
        $this->select('jform_itemId', 'value='.$itemId);
    }

    private function selectRouteByPath($path) {
        if ($this->getSelectedLabel('limit') != 'All') {
            $this->select('limit', 'All');
            $this->waitForPageToLoad('30000');
        }
        $this->click('//table/tbody//tr/td/a[normalize-space(text()) = "'.$path.'"]/../../td[position() = 2]/input');
    }

    private function testParse($path) {
        $this->type('path', $path);
        $this->click('//form[@id="test-path-form"]//input[@type="submit"]');
        $this->waitForPageToLoad('30000');
    }

    private function testBuild($query) {
        $this->type('query', $query);
        $this->click('//form[@id="test-query-form"]//input[@type="submit"]');
        $this->waitForPageToLoad('30000');
    }

    private function sortByColumnHeader($column) {
        $this->click('//table/thead//tr/th/a[normalize-space(text()) = "'.$column.'"]');
        $this->waitForPageToLoad('30000');

        if ($this->getSelectedLabel('limit') != 'All') {
            $this->select('limit', 'All');
            $this->waitForPageToLoad('30000');
        }
    }

    private function assertPathBeforePath($path1, $path2, $message) {
        $this->assertTrue($this->getRouteIndexByPath($path1) < $this->getRouteIndexByPath($path2), $message);
    }

    private function getRouteIndexByPath($path) {
        return $this->getElementIndex('//table/tbody//tr/td/a[normalize-space(text()) = "'.$path.'"]/../..');
    }

    private function sortByToolbar($order) {
        $this->select('sortTable', $order);
        $this->waitForPageToLoad(30000);
    }

    private function sortByToolbarDirection($direction) {
        $this->select('directionTable', $direction);
        $this->waitForPageToLoad(30000);
    }

    private function getRouteIdByPath($path) {
        if ($this->getSelectedLabel('limit') != 'All') {
            $this->select('limit', 'All');
            $this->waitForPageToLoad('30000');
        }
        return $this->getText('//table/tbody//tr/td/a[normalize-space(text()) = "'.$path.'"]/../../td[position() = 1]');
    }

    private function getRouteQueryByPath($path) {
        if ($this->getSelectedLabel('limit') != 'All') {
            $this->select('limit', 'All');
            $this->waitForPageToLoad('30000');
        }
        return $this->getText('//table/tbody//tr/td/a[normalize-space(text()) = "'.$path.'"]/../../td[position() = 4]');
    }

    private function getNumberOfRoutes() {
        return $this->getXpathCount('//table/tbody//tr');
    }
}
