<?php
/**
 * @copyright  Copyright (C) 2012 Daniel Calviño Sánchez
 * @copyright  Copyright (C) 2014 Daniel Calviño Sánchez
 * @copyright  Copyright (C) 2016 Daniel Calviño Sánchez
 * @license    GNU Affero General Public License version 3 or later; see LICENSE.txt
 */

require_once JPATH_BASE.'/tests/unit/core/case/database.php';
require_once 'PHPUnit/Extensions/Database/DataSet/CsvDataSet.php';

require_once JPATH_BASE.'/plugins/system/simplecustomrouter/simplecustomrouter.php';
require_once JPATH_BASE.'/libraries/joomla/uri/uri.php';
require_once JPATH_BASE.'/libraries/legacy/request/request.php';

class SimpleCustomRouterTest extends TestCaseDatabase {

    private $prefix;
    private $siteRouter;

    private $simpleCustomRouter;

    public static function setUpBeforeClass() {
        parent::setUpBeforeClass();

        //The default database schema loaded by the parent class
        //(JPATH_TESTS . '/schema/ddl.sql') does not create the table
        //'jos_simplecustomrouter'. The SQL file used here creates that missing
        //table.
        $pdo = self::$driver->getConnection();
        $pdo->exec(file_get_contents(JPATH_BASE . '/tests/unit/suites/plugins/system/simplecustomrouter/create-test-tables.sql'));
    }

    protected function setUp() {
        parent::setUp();

        $_SERVER['HTTP_HOST'] = 'localhost';

        JFactory::$application = $this->getMockApplication();

        $this->siteRouter = null;

        $this->simpleCustomRouter = new SimpleCustomRouter();
    }

    protected function getDataSet() {
        $prefix = JFactory::$database->getPrefix();

        $dataSet = new PHPUnit_Extensions_Database_DataSet_CsvDataSet(',', "'", '\\');
        $dataSet->addTable($prefix.'simplecustomrouter', JPATH_BASE . '/tests/unit/suites/plugins/system/simplecustomrouter/simplecustomrouter.csv');
        return $dataSet;
    }

    public function testParsePath() {
        $this->assertParsedPath('path', '', 'option=parsePath', '');
        $this->assertParsedPath('pathLongerThanPrevious', '', 'option=parsePathLonger', '');
        $this->assertParsedPath('pathEvenLongerThanPrevious', '', 'option=parsePathLonger&grade=longest', '');
    }

    public function testParseComposedPath() {
        $this->assertParsedPath('path/childComponent/grandChildComponent', '', 'option=parsePath&type=composed', '');
    }

    public function testParsePathAssociatedToItemid() {
        $this->assertParsedPath('pathAssociatedToItemid', '', 'option=parsePathAssociatedToItemid', '42');
    }

    public function testParsePathWithRegularExpressionCharacters() {
        $this->assertParsedPath('path.with.regularExpressionCharacters', '', 'option=parsePathWithRegularExpressionCharacters', '');
        $this->assertParsedPath('path-with.regularExpressionCharacters', 'path-with.regularExpressionCharacters', '', '');
        $this->assertParsedPath('path.with-regularExpressionCharacters', 'path.with-regularExpressionCharacters', '', '');
    }

    public function testParsePathWithOneParameter() {
        $this->assertParsedPath('parametrizedPath-4', '', 'option=parseParametrizedPath&id=4', '');
        $this->assertParsedPath('parametrizedPath-8', '', 'option=parseParametrizedPath&id=8', '');
    }

    public function testParsePathWithSeveralParameters() {
        $this->assertParsedPath('parametrizedPath-15/anotherParameter-yes', '', 'option=parseParametrizedPath&id=15&extra=yes', '');
        $this->assertParsedPath('parametrizedPath-16/anotherParameter-no', '', 'option=parseParametrizedPath&id=16&extra=no', '');
    }

    public function testParsePathWithSeveralParametersInDifferentOrder() {
        $this->assertParsedPath('parametrizedPathInDifferentOrder-23/anotherParameter-yes', '', 'option=parseParametrizedPathInDifferentOrder&extra=yes&id=23', '');
        $this->assertParsedPath('parametrizedPathInDifferentOrder-42/anotherParameter-no', '', 'option=parseParametrizedPathInDifferentOrder&extra=no&id=42', '');
    }

    public function testParsePathWithSeveralParametersAndRegularExpressionCharacters() {
        $this->assertParsedPath('parametrizedPath.withRegularExpressionCharacters-15/another.parameter-yes', '', 'option=parseParametrizedPathWithRegularExpressionCharacters&id=15&extra=yes', '');
        $this->assertParsedPath('parametrizedPath-withRegularExpressionCharacters-15/another.parameter-yes', 'parametrizedPath-withRegularExpressionCharacters-15/another.parameter-yes', '', '');
        $this->assertParsedPath('parametrizedPath.withRegularExpressionCharacters-15/another-parameter-yes', 'parametrizedPath.withRegularExpressionCharacters-15/another-parameter-yes', '', '');
    }

    public function testParsePathWithExtraQuery() {
        $this->assertParsedPathAndQuery('path', 'field1=value1&field2=value2', '', 'option=parsePath&field1=value1&field2=value2', '');
    }

    public function testParsePathWithItemidAsExtraQuery() {
        $this->assertParsedPathAndQuery('path', 'Itemid=108', '', 'option=parsePath', '');
        $this->assertParsedPathAndQuery('path', 'Itemid=108&field1=value1&field2=value2', '', 'option=parsePath&field1=value1&field2=value2', '');
        $this->assertParsedPathAndQuery('path', 'field1=value1&Itemid=108&field2=value2', '', 'option=parsePath&field1=value1&field2=value2', '');
        $this->assertParsedPathAndQuery('path', 'field1=value1&field2=value2&Itemid=108', '', 'option=parsePath&field1=value1&field2=value2', '');
        $this->assertParsedPathAndQuery('pathAssociatedToItemid', 'Itemid=108', '', 'option=parsePathAssociatedToItemid', '42');
    }

    public function testParseComplexPath() {
        $this->assertParsedPathAndQuery('complexPath-4/anotherParameter-good/anotherComponent', 'field1=value1&Itemid=42&field2=value2', '', 'option=parseComplexPath&extra=good&id=4&field1=value1&field2=value2', '108');
        $this->assertParsedPathAndQuery('complexPath-8/anotherParameter-bad/anotherComponent', 'field1=value1&field2=value2&Itemid=42', '', 'option=parseComplexPath&extra=bad&id=8&field1=value1&field2=value2', '108');
    }

    public function testParseDuplicatedPath() {
        $this->assertParsedPath('duplicatedPath', '', 'option=parseDuplicatedPath1', '');
        $this->assertParsedPath('duplicatedPath', '', 'option=parseDuplicatedPath1', '');
    }

    public function testParseUnknownPath() {
        $this->assertParsedPath('unknownPath', 'unknownPath', '', '');
    }

    public function testParsePartialPath() {
        $this->assertParsedPath('path/otherPathPartNotHandled', 'path/otherPathPartNotHandled', '', '');
    }

    public function testParseParametrizedPathNotMatching() {
        $this->assertParsedPath('parametrizedPath-wordInsteadOfNumber', 'parametrizedPath-wordInsteadOfNumber', '', '');
    }

    private function assertParsedPath($path, $expectedPath, $expectedQuery, $expectedItemId) {
        //Ensure that the request does not contain a value from a previous
        //parsing
        JFactory::getApplication()->input->set('Itemid', null);

        $uri = new JURI();
        $uri->setPath($path);

        $this->assertEquals(0, count($this->simpleCustomRouter->parse($this->siteRouter, $uri)), 'Returned vars');
        $this->assertEquals($expectedPath, $uri->getPath(), 'Parsed path');
        $this->assertEquals($expectedQuery, $uri->getQuery(), 'Parsed query');
        $this->assertEquals($expectedItemId, JFactory::getApplication()->input->get('Itemid'), 'Assigned Itemid');
    }

    private function assertParsedPathAndQuery($path, $query, $expectedPath, $expectedQuery, $expectedItemId) {
        //Ensure that the request does not contain a value from a previous
        //parsing, and set the Itemid value from the query, if any.
        parse_str($query, $queryArray);
        if (isset($queryArray['Itemid'])) {
            JFactory::getApplication()->input->set('Itemid', $queryArray['Itemid']);
        } else {
            JFactory::getApplication()->input->set('Itemid', null);
        }

        $uri = new JURI();
        $uri->setPath($path);
        $uri->setQuery($query);

        $this->assertEquals(0, count($this->simpleCustomRouter->parse($this->siteRouter, $uri)), 'Returned vars');
        $this->assertEquals($expectedPath, $uri->getPath(), 'Parsed path');
        $this->assertEquals($expectedQuery, $uri->getQuery(), 'Parsed query');
        $this->assertEquals($expectedItemId, JFactory::getApplication()->input->get('Itemid'), 'Assigned Itemid');
    }

    public function testBuildRoute() {
        $this->assertBuildRoute('option=buildRoute', 'route', '');
        $this->assertBuildRoute('option=buildRouteLonger', 'route/longerThanPrevious', '');
    }

    public function testBuildComposedRoute() {
        $this->assertBuildRoute('option=buildComposedRoute&someField=someValue&anotherField=anotherValue', 'composedRoute', '');
        $this->assertBuildRoute('option=buildComposedRoute&anotherField=anotherValue&someField=someValue', 'composedRoute', '');
        $this->assertBuildRoute('anotherField=anotherValue&option=buildComposedRoute&someField=someValue', 'composedRoute', '');
        $this->assertBuildRoute('anotherField=anotherValue&someField=someValue&option=buildComposedRoute', 'composedRoute', '');
        $this->assertBuildRoute('someField=someValue&anotherField=anotherValue&option=buildComposedRoute', 'composedRoute', '');
        $this->assertBuildRoute('someField=someValue&option=buildComposedRoute&anotherField=anotherValue', 'composedRoute', '');
    }

    public function testBuildRouteWithLessFields() {
        //Before composedRoute in the database
        $this->assertBuildRoute('option=buildComposedRoute&someField=someValue', 'routeWithLessFields', '');
        //After composedRoute in the database
        $this->assertBuildRoute('option=buildComposedRoute&anotherField=anotherValue', 'anotherRouteWithLessFields', '');
    }

    public function testBuildRouteWithItemid() {
        $this->assertBuildRoute('option=buildRouteAssociatedToItemid', 'routeAssociatedToItemid', '');
        $this->assertBuildRoute('option=buildRouteAssociatedToItemid&Itemid=108', 'routeAssociatedToItemid', '');
        $this->assertBuildRoute('option=buildRoute&Itemid=108', 'route', '');
    }

    public function testBuildRouteWithRegularExpressionCharacters() {
        $this->assertBuildRoute('option=buildRoute.with.regularExpressionCharacters', 'routeWithRegularExpressionCharacters', '');
        $this->assertBuildRoute('option=buildRoute-with.regularExpressionCharacters', 'index.php', 'option=buildRoute-with.regularExpressionCharacters');
        $this->assertBuildRoute('option=buildRoute.with-regularExpressionCharacters', 'index.php', 'option=buildRoute.with-regularExpressionCharacters');
    }

    public function testBuildRouteWithOneParameter() {
        $this->assertBuildRoute('option=buildParametrizedRoute&id=4', 'parametrizedRoute-4', '');
        $this->assertBuildRoute('option=buildParametrizedRoute&id=8', 'parametrizedRoute-8', '');
    }

    public function testBuildRouteWithSeveralParameters() {
        $this->assertBuildRoute('option=buildParametrizedRoute&id=15&extra=yes', 'parametrizedRoute-15/anotherParameter-yes', '');
        $this->assertBuildRoute('option=buildParametrizedRoute&extra=yes&id=15', 'parametrizedRoute-15/anotherParameter-yes', '');
        $this->assertBuildRoute('option=buildParametrizedRoute&id=16&extra=no', 'parametrizedRoute-16/anotherParameter-no', '');
        $this->assertBuildRoute('option=buildParametrizedRoute&extra=no&id=16', 'parametrizedRoute-16/anotherParameter-no', '');
    }

    public function testBuildRouteWithSeveralParametersInDifferentOrder() {
        $this->assertBuildRoute('option=buildParametrizedRouteInDifferentOrder&extra=yes&id=23', 'parametrizedRouteInDifferentOrder-23/anotherParameter-yes', '');
        $this->assertBuildRoute('option=buildParametrizedRouteInDifferentOrder&id=23&extra=yes', 'parametrizedRouteInDifferentOrder-23/anotherParameter-yes', '');
        $this->assertBuildRoute('option=buildParametrizedRouteInDifferentOrder&extra=no&id=42', 'parametrizedRouteInDifferentOrder-42/anotherParameter-no', '');
        $this->assertBuildRoute('option=buildParametrizedRouteInDifferentOrder&id=42&extra=no', 'parametrizedRouteInDifferentOrder-42/anotherParameter-no', '');
    }

    public function testBuildRouteWithSeveralParametersInTheSameField() {
        $this->assertBuildRoute('option=buildParametrizedRoute&fieldWithSeveralParameters=John-and-Jack', 'parametrizedRoute-John-Jack', '');
        $this->assertBuildRoute('option=buildParametrizedRoute&fieldWithSeveralParameters=Jack-and-John', 'parametrizedRoute-Jack-John', '');
        $this->assertBuildRoute('option=buildParametrizedRoute&fieldWithSeveralParameters=4-and-John', 'parametrizedRoute-4-John', '');
        $this->assertBuildRoute('option=buildParametrizedRoute&fieldWithSeveralParameters=John-and-4', 'index.php', 'option=buildParametrizedRoute&fieldWithSeveralParameters=John-and-4');
    }

    public function testBuildRouteWithSeveralParametersAndRegularExpressionCharacters() {
        $this->assertBuildRoute('option=buildParametrizedRouteWithRegularExpressionCharacters&id=8&extra=yes.and.no', 'parametrizedRouteWithRegularExpressionCharacters-8/anotherParameter-yes-and-no', '');
        $this->assertBuildRoute('option=buildParametrizedRouteWithRegularExpressionCharacters&id=8&extra=yes-and-no', 'index.php', 'option=buildParametrizedRouteWithRegularExpressionCharacters&id=8&extra=yes-and-no');
    }

    public function testBuildExplicitRouteThatMatchesParametrizedRoute() {
        $this->assertBuildRoute('option=buildParametrizedRoute&id=42', 'explicitRouteThatMatchesParametrizedRoute', '');
    }

    public function testBuildRouteWithSomeUnknownField() {
        $this->assertBuildRoute('option=buildRoute&unknownField1=value1&unknownField2=value2', 'route', 'unknownField1=value1&unknownField2=value2');
    }

    public function testBuildComplexRoute() {
        $this->assertBuildRoute('unknownField1=value1&option=buildComplexRoute&Itemid=42&id=4&unknownField2=value2', 'complexRoute-4/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&option=buildComplexRoute&Itemid=42&id=8&unknownField2=value2', 'complexRoute-8/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&extra=yes&option=buildComplexRoute&Itemid=42&id=4&unknownField2=value2', 'complexRoute-4/anotherParameter-yes/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&extra=no&option=buildComplexRoute&Itemid=42&id=8&unknownField2=value2', 'complexRoute-8/anotherParameter-no/anotherComponent', 'unknownField1=value1&unknownField2=value2');
    }

    public function testBuildRouteWithRepeatedFields() {
        $this->assertBuildRoute('option=buildComplexRoute&id=4&extra=yes&option=buildParametrizedRoute&extra=no', 'parametrizedRoute-4/anotherParameter-no', '');
    }

    public function testBuildDuplicatedRoute() {
        $this->assertBuildRoute('option=buildDuplicatedRoute', 'duplicatedRoute1', '');
        $this->assertBuildRoute('option=buildDuplicatedRoute', 'duplicatedRoute1', '');
    }

    public function testBuildUnknownRoute() {
        $this->assertBuildRoute('option=buildUnknownRoute', 'index.php', 'option=buildUnknownRoute');
    }

    public function testBuildUnknownRouteWithItemid() {
        $this->assertBuildRoute('option=buildUnknownRoute&Itemid=108', 'index.php', 'option=buildUnknownRoute&Itemid=108');
    }

    public function testBuildParametrizedRouteNotMatching() {
        $this->assertBuildRoute('option=buildParametrizedRoute&id=wordInsteadOfNumber', 'index.php', 'option=buildParametrizedRoute&id=wordInsteadOfNumber');
    }

    private function assertBuildRoute($route, $expectedPath, $expectedQuery) {
        $uri = new JURI();
        $uri->setPath('index.php');
        $uri->setQuery($route);

        $this->assertEquals($uri, $this->simpleCustomRouter->build($this->siteRouter, $uri), 'Returned uri');
        $this->assertEquals($expectedPath, $uri->getPath(), 'Built path');
        $this->assertEquals($expectedQuery, $uri->getQuery(), 'Built query');
    }

    public function testParseComplexPathAndBuildComplexRoute() {
        //As routes are loaded just once from the database and reused for both
        //parsing and building this test ensures that the changes made in the
        //loaded routes when parsing and when building do not interfere in each
        //other.

        $this->assertParsedPathAndQuery('complexPath-4/anotherParameter-good/anotherComponent', 'field1=value1&Itemid=42&field2=value2', '', 'option=parseComplexPath&extra=good&id=4&field1=value1&field2=value2', '108');

        $this->assertBuildRoute('unknownField1=value1&option=buildComplexRoute&Itemid=42&id=4&unknownField2=value2', 'complexRoute-4/anotherComponent', 'unknownField1=value1&unknownField2=value2');

        $this->assertParsedPathAndQuery('complexPath-8/anotherParameter-bad/anotherComponent', 'field1=value1&field2=value2&Itemid=42', '', 'option=parseComplexPath&extra=bad&id=8&field1=value1&field2=value2', '108');

        $this->assertBuildRoute('unknownField1=value1&extra=no&option=buildComplexRoute&Itemid=42&id=8&unknownField2=value2', 'complexRoute-8/anotherParameter-no/anotherComponent', 'unknownField1=value1&unknownField2=value2');
    }

    public function testParsePathWithCache() {
        //Ensure that no cache instances exist
        JFactory::$cache = null;
        jimport('joomla.cache.cache');
        JCache::$_handler = array();

        //Ensure that the cache has a proper configuration
        JFactory::getConfig()->set('caching', '1');
        JFactory::getConfig()->set('cache_handler', 'TestStub');
        JFactory::getConfig()->set('cachetime', '15');

        $cache = JFactory::getCache('simplecustomrouter', '');
        $cacheStorage = $cache->_getStorage();
        $this->simpleCustomRouter = new SimpleCustomRouter($cache);

        //Ensure that the cache is empty
        $this->assertEquals(0, count($cacheStorage->data), 'Cache count');

        $this->assertParsedPathAndQuery('complexPath-4/anotherParameter-good/anotherComponent', 'field1=value1&Itemid=42&field2=value2', '', 'option=parseComplexPath&extra=good&id=4&field1=value1&field2=value2', '108');
        $this->assertParsedPathAndQuery('complexPath-8/anotherParameter-bad/anotherComponent', 'field1=value1&field2=value2&Itemid=42', '', 'option=parseComplexPath&extra=bad&id=8&field1=value1&field2=value2', '108');
        //Test also a route without an Itemid to ensure that the Itemid in the
        //request is cleared when the route is got from the cache.
        $this->assertParsedPathAndQuery('path', 'Itemid=42', '', 'option=parsePath', '');

        //Check that the routes were cached
        $key1 = 'parse: complexPath-4/anotherParameter-good/anotherComponent';
        $this->assertTrue(array_key_exists($key1, $cacheStorage->data), 'Key1');
        $this->assertEquals(array('option=parseComplexPath&extra=good&id=4&field1=value1&field2=value2', '108'), unserialize($cacheStorage->data[$key1]), 'Key1 stored data');

        $key2 = 'parse: complexPath-8/anotherParameter-bad/anotherComponent';
        $this->assertTrue(array_key_exists($key2, $cacheStorage->data), 'Key2');
        $this->assertEquals(array('option=parseComplexPath&extra=bad&id=8&field1=value1&field2=value2', '108'), unserialize($cacheStorage->data[$key2]), 'Key2 stored data');

        $key3 = 'parse: path';
        $this->assertTrue(array_key_exists($key3, $cacheStorage->data), 'Key3');
        $this->assertEquals(array('option=parsePath', '0'), unserialize($cacheStorage->data[$key3]), 'Key3 stored data');

        //Check that the cached routes were not gotten yet
        $this->assertEquals(0, $cacheStorage->accessedDataCount[$key1], 'Key1 access count first use');
        $this->assertEquals(0, $cacheStorage->accessedDataCount[$key2], 'Key2 access count first use');
        $this->assertEquals(0, $cacheStorage->accessedDataCount[$key3], 'Key3 access count first use');

        //Parse the routes again to use the cache
        $this->assertParsedPathAndQuery('complexPath-4/anotherParameter-good/anotherComponent', 'Itemid=42&field1=value1&field2=value2', '', 'option=parseComplexPath&extra=good&id=4&field1=value1&field2=value2', '108');
        $this->assertParsedPathAndQuery('complexPath-4/anotherParameter-good/anotherComponent', 'field1=value1&Itemid=42&field2=value2', '', 'option=parseComplexPath&extra=good&id=4&field1=value1&field2=value2', '108');
        $this->assertParsedPathAndQuery('complexPath-4/anotherParameter-good/anotherComponent', 'field1=value1&field2=value2&Itemid=42', '', 'option=parseComplexPath&extra=good&id=4&field1=value1&field2=value2', '108');
        $this->assertParsedPathAndQuery('complexPath-8/anotherParameter-bad/anotherComponent', 'Itemid=42&field1=value1&field2=value2', '', 'option=parseComplexPath&extra=bad&id=8&field1=value1&field2=value2', '108');
        $this->assertParsedPathAndQuery('complexPath-8/anotherParameter-bad/anotherComponent', 'field1=value1&Itemid=42&field2=value2', '', 'option=parseComplexPath&extra=bad&id=8&field1=value1&field2=value2', '108');
        $this->assertParsedPathAndQuery('complexPath-8/anotherParameter-bad/anotherComponent', 'field1=value1&field2=value2&Itemid=42', '', 'option=parseComplexPath&extra=bad&id=8&field1=value1&field2=value2', '108');
        $this->assertParsedPathAndQuery('path', 'Itemid=42', '', 'option=parsePath', '');
        $this->assertParsedPathAndQuery('path', 'Itemid=42', '', 'option=parsePath', '');
        $this->assertParsedPathAndQuery('path', 'Itemid=42', '', 'option=parsePath', '');

        //Check that the routes were accessed 3 times each one
        $this->assertEquals(3, $cacheStorage->accessedDataCount[$key1], 'Key1 access count several uses');
        $this->assertEquals(3, $cacheStorage->accessedDataCount[$key2], 'Key2 access count several uses');
        $this->assertEquals(3, $cacheStorage->accessedDataCount[$key3], 'Key3 access count several uses');

        //Check that the routes were stored just once
        $this->assertEquals(1, $cacheStorage->storedDataCount[$key1], 'Key1 stored count several uses');
        $this->assertEquals(1, $cacheStorage->storedDataCount[$key2], 'Key2 stored count several uses');
        $this->assertEquals(1, $cacheStorage->storedDataCount[$key3], 'Key3 stored count several uses');
    }

    public function testBuildRouteWithCache() {
        //Ensure that no cache instances exist
        JFactory::$cache = null;
        jimport('joomla.cache.cache');
        JCache::$_handler = array();

        //Ensure that the cache has a proper configuration
        JFactory::getConfig()->set('caching', '1');
        JFactory::getConfig()->set('cache_handler', 'TestStub');
        JFactory::getConfig()->set('cachetime', '15');

        $cache = JFactory::getCache('simplecustomrouter', '');
        $cacheStorage = $cache->_getStorage();
        $this->simpleCustomRouter = new SimpleCustomRouter($cache);

        //Ensure that the cache is empty
        $this->assertEquals(0, count($cacheStorage->data), 'Cache count');

        $this->assertBuildRoute('unknownField1=value1&extra=yes&option=buildComplexRoute&Itemid=42&id=4&unknownField2=value2', 'complexRoute-4/anotherParameter-yes/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&extra=no&option=buildComplexRoute&Itemid=42&id=8&unknownField2=value2', 'complexRoute-8/anotherParameter-no/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        //Test also a route without an Itemid to ensure that the Itemid in the
        //query is removed when the route is got from the cache.
        $this->assertBuildRoute('option=buildRoute&Itemid=42', 'route', '');

        //Check that the routes were cached
        $key1 = 'build: unknownField1=value1&extra=yes&option=buildComplexRoute&Itemid=42&id=4&unknownField2=value2';
        $this->assertTrue(array_key_exists($key1, $cacheStorage->data), 'Key1');
        $this->assertEquals(array('complexRoute-4/anotherParameter-yes/anotherComponent', 'unknownField1=value1&unknownField2=value2'), unserialize($cacheStorage->data[$key1]), 'Key1 stored data');

        $key2 = 'build: unknownField1=value1&extra=no&option=buildComplexRoute&Itemid=42&id=8&unknownField2=value2';
        $this->assertTrue(array_key_exists($key2, $cacheStorage->data), 'Key2');
        $this->assertEquals(array('complexRoute-8/anotherParameter-no/anotherComponent', 'unknownField1=value1&unknownField2=value2'), unserialize($cacheStorage->data[$key2]), 'Key2 stored data');

        $key3 = 'build: option=buildRoute&Itemid=42';
        $this->assertTrue(array_key_exists($key3, $cacheStorage->data), 'Key3');
        $this->assertEquals(array('route', ''), unserialize($cacheStorage->data[$key3]), 'Key3 stored data');

        //Check that the cached routes were not gotten yet
        $this->assertEquals(0, $cacheStorage->accessedDataCount[$key1], 'Key1 access count first use');
        $this->assertEquals(0, $cacheStorage->accessedDataCount[$key2], 'Key2 access count first use');
        $this->assertEquals(0, $cacheStorage->accessedDataCount[$key3], 'Key3 access count first use');

        //Parse the routes again to use the cache
        $this->assertBuildRoute('unknownField1=value1&extra=yes&option=buildComplexRoute&Itemid=42&id=4&unknownField2=value2', 'complexRoute-4/anotherParameter-yes/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&extra=yes&option=buildComplexRoute&Itemid=42&id=4&unknownField2=value2', 'complexRoute-4/anotherParameter-yes/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&extra=yes&option=buildComplexRoute&Itemid=42&id=4&unknownField2=value2', 'complexRoute-4/anotherParameter-yes/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&extra=no&option=buildComplexRoute&Itemid=42&id=8&unknownField2=value2', 'complexRoute-8/anotherParameter-no/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&extra=no&option=buildComplexRoute&Itemid=42&id=8&unknownField2=value2', 'complexRoute-8/anotherParameter-no/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('unknownField1=value1&extra=no&option=buildComplexRoute&Itemid=42&id=8&unknownField2=value2', 'complexRoute-8/anotherParameter-no/anotherComponent', 'unknownField1=value1&unknownField2=value2');
        $this->assertBuildRoute('option=buildRoute&Itemid=42', 'route', '');
        $this->assertBuildRoute('option=buildRoute&Itemid=42', 'route', '');
        $this->assertBuildRoute('option=buildRoute&Itemid=42', 'route', '');

        //Check that the routes were accessed 3 times each one
        $this->assertEquals(3, $cacheStorage->accessedDataCount[$key1], 'Key1 access count several uses');
        $this->assertEquals(3, $cacheStorage->accessedDataCount[$key2], 'Key2 access count several uses');
        $this->assertEquals(3, $cacheStorage->accessedDataCount[$key3], 'Key3 access count several uses');

        //Check that the routes were stored just once
        $this->assertEquals(1, $cacheStorage->storedDataCount[$key1], 'Key1 stored count several uses');
        $this->assertEquals(1, $cacheStorage->storedDataCount[$key2], 'Key2 stored count several uses');
        $this->assertEquals(1, $cacheStorage->storedDataCount[$key3], 'Key3 stored count several uses');
    }

    public function testParsePathWithCachePerformance() {
        $this->markTestSkipped('Cache performance test can yield false positives');

        //It is not really a single operation, but anyway most of the time is
        //spent building the routes.
        $this->cachePerformanceTest('testParseComplexPath');
    }

    public function testBuildRouteWithCachePerformance() {
        $this->markTestSkipped('Cache performance test can yield false positives');

        //It is not really a single operation, but anyway most of the time is
        //spent building the routes.
        $this->cachePerformanceTest('testBuildComplexRoute');
    }

    /**
     * Performance testing helper.
     * It is not used in any automated tests as it can yield false positives.
     * Anyway, its best use is with build tests, as their operations are heavier
     * and benefit a lot from cache (thus reducing the risk of false positives).
     * Parse tests, on the other hand, are less affected by cache and may yield
     * false positives more frequently.
     */
    private function cachePerformanceTest($methodName) {
        //Ensure that no cache instances exist
        JFactory::$cache = null;

        //Ensure that the cache has a proper configuration
        JFactory::getConfig()->set('cache_handler', 'file');
        JFactory::getConfig()->set('cachetime', '15');

        $cache = JFactory::getCache('simplecustomrouter', '');
        $this->simpleCustomRouter = new SimpleCustomRouter($cache);

        $cache->setCaching(false);

        //Check how long it takes to do a single operation.
        $singleOperationTime = $this->elapsedTimeToExecute(array($this, $methodName), 1);

        //Repeat 100 times
        $repeatedOperationTime = $this->elapsedTimeToExecute(array($this, $methodName), 100);

        //Ensure that, without cache, executing the operation 100 times is at
        //least 40 times slower than executing a single operation (it will not
        //be more than 100 times slower due to other optimizations in the
        //system that can not be controlled here). 40 times is just a somewhat
        //arbitrary threshold; note that this test may fail yet the cache can be
        //disabled.
        $this->assertGreaterThan($singleOperationTime*40, $repeatedOperationTime, 'Time with cache disabled (it can be a false positive)');

        $cache->setCaching(true);

        //Repeat 100 times, now with the cache enabled
        $repeatedOperationTime = $this->elapsedTimeToExecute(array($this, $methodName), 100);

        //Ensure that, with cache, executing the operation 100 times is not more
        //than 20 times slower than executing a single operation. 20 times is
        //just a somewhat arbitrary threshold; note that this test may fail yet
        //the cache can be working.
        $this->assertLessThan($singleOperationTime*20, $repeatedOperationTime, 'Time with cache enabled (it can be a false positive)');
    }

    private function elapsedTimeToExecute($callback, $times) {
        $startTime = microtime(true);
        for ($i=0; $i<$times; $i++) {
            //The penalization of using call_user_func_array is negligible
            //compared to the time spent in the functions that will be called
            //in the tests
            call_user_func_array($callback, array());
        }
        $endTime = microtime(true);

        return $endTime - $startTime;
    }
}

require_once JPATH_BASE.'/libraries/joomla/cache/storage.php';

/**
 * Cache storage to be used in cache tests.
 */
class JCacheStorageTestStub extends JCacheStorage {

    public $data = array();
    public $accessedDataCount = array();
    public $storedDataCount = array();

    public function get($id, $group, $checkTime = true) {
        if (array_key_exists($id, $this->data)) {
            $this->accessedDataCount[$id]++;
            return $this->data[$id];
        }

        return false;
    }

    public function store($id, $group, $data) {
        if (array_key_exists($id, $this->data)) {
            $this->storedDataCount[$id]++;
        } else {
            $this->storedDataCount[$id] = 1;
            $this->accessedDataCount[$id] = 0;
        }
        $this->data[$id] = $data;

        return true;
    }

    //Needed for unlock fallback in JCache
    public function remove($id, $group) {
        unset($this->data[$id]);
        return true;
    }
}
